package cn.iocoder.yudao.module.ecw.dal.mysql.currencyRate;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currencyRate.CurrencyRateDO;
import cn.iocoder.yudao.module.ecw.vo.currencyRate.CurrencyRateQueryParamVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.ibatis.annotations.Mapper;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;

@Mapper
public interface CurrencyRateMapper extends AbstractMapper<CurrencyRateDO> {
    default PageResult<CurrencyRateDO> selectPage(CurrencyRateQueryParamVO param) {
        Date now = new Date();
        List<Long> countries = Optional.ofNullable(param.getCountries()).orElse(Collections.emptyList());
        Consumer<LambdaQueryWrapper<CurrencyRateDO>> countriesQuery = it ->
                countries.stream().limit(5).forEach(id ->
                        it.or().apply("json_contains(`countries`, {0})", String.valueOf(id)));
        LambdaQueryWrapper<CurrencyRateDO> wrapper = Wrappers.lambdaQuery(CurrencyRateDO.class)
                .eq(param.getSourceId() != null, CurrencyRateDO::getSourceId, param.getSourceId())
                .eq(param.getTargetId() != null, CurrencyRateDO::getTargetId, param.getTargetId())
                .and(!countries.isEmpty(), countriesQuery)
                .gt(param.getExpired() == Boolean.FALSE, CurrencyRateDO::getExpiration, now)
                .lt(param.getExpired() == Boolean.TRUE, CurrencyRateDO::getExpiration, now)
                .gt(param.getExpirationAfter() != null, CurrencyRateDO::getExpiration, param.getExpirationAfter())
                .lt(param.getExpirationBefore() != null, CurrencyRateDO::getExpiration, param.getExpirationBefore())
                .orderByDesc(Arrays.asList(CurrencyRateDO::getUpdateTime, CurrencyRateDO::getExpiration));
        return selectPage(param, wrapper);
    }

   default List<CurrencyRateDO> allAvailable() {
       Date now = new Date();
       LambdaQueryWrapper<CurrencyRateDO> wrapper = Wrappers.lambdaQuery(CurrencyRateDO.class)
               .gt( CurrencyRateDO::getExpiration, now)
               .orderByDesc(CurrencyRateDO::getId);
       return selectList(wrapper);
   }

}
